@@ -4,7 +4,7 @@
 
  library for simulation of random fields -- get key strukture
 
- Copyright (C) 2001 -- 2015 Martin Schlather, 
+ Copyright (C) 2001 -- 2017 Martin Schlather, 
 
 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
@@ -21,19 +21,17 @@ along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
 
-//#include <math.h>  
+//#include <Rmath.h>  
 #include <stdio.h>  
-#include <stdlib.h>
+//#include <stdlib.h>
 //#include <sys/timeb.h>
- 
-#include <string.h>
-#include "RF.h"
-#include <Rdefines.h>
-#include "Coordinate_systems.h"
-
-//#include "CovFcts.h"
 //#include <unistd.h>
 //#include <R_ext/Utils.h>     
+#include <string.h>
+#include <Rdefines.h>
+#include "RF.h"
+#include "Coordinate_systems.h"
+
 
 
 #define WHICH_USER 0
@@ -355,7 +353,7 @@ SEXP GetModelInfo(cov_model *cov, int prlevel, bool both, int spConform,
 
   if (spConform) {
     char name[MAXCHAR+2];
-    sprintf(name, "%s", CC->nick); 
+    SPRINTF(name, "%s", CC->nick); 
     SET_VECTOR_ELT(model, k++, mkString(name));
   } else {
     SET_VECTOR_ELT(model, k++, mkString(CC->name));
@@ -368,7 +366,7 @@ SEXP GetModelInfo(cov_model *cov, int prlevel, bool both, int spConform,
 
       if (cov->kappasub[i] != NULL) {	  
 	  SET_STRING_ELT(pnames, j, 
-			 mkChar(!strcmp(C->kappanames[i], FREEVARIABLE)
+			 mkChar(!STRCMP(C->kappanames[i], FREEVARIABLE)
 				&& cov->ownkappanames[i] != NULL
 				? cov->ownkappanames[i] 
 				: C->kappanames[i])); 
@@ -401,7 +399,7 @@ SEXP GetModelInfo(cov_model *cov, int prlevel, bool both, int spConform,
 	}
 	
 	SET_STRING_ELT(pnames, j, 
-		       mkChar(!strcmp(C->kappanames[i], FREEVARIABLE)
+		       mkChar(!STRCMP(C->kappanames[i], FREEVARIABLE)
 			      && cov->ownkappanames[i] != NULL
 			      ? cov->ownkappanames[i] 
 			      : C->kappanames[i]));
@@ -590,10 +588,10 @@ SEXP GetExtModelInfo(SEXP keynr, SEXP Prlevel, SEXP spConform, SEXP whichSub) {
   int knr = INTEGER(keynr)[0],
     which = INTEGER(whichSub)[0] == WHICH_ALL_USER 
     ? WHICH_ALL : INTEGER(whichSub)[0] % 2,
-    prlevel = abs(INTEGER(Prlevel)[0]) % 10;
+    prlevel = std::abs(INTEGER(Prlevel)[0]) % 10;
   bool 
     both = INTEGER(Prlevel)[0] < 0,
-    delete_call = abs(INTEGER(Prlevel)[0]) < 10;
+    delete_call = std::abs(INTEGER(Prlevel)[0]) < 10;
   cov_model *cov, *orig;
   SEXP res, names;
 
@@ -607,7 +605,7 @@ SEXP GetExtModelInfo(SEXP keynr, SEXP Prlevel, SEXP spConform, SEXP whichSub) {
       int i, len = length(names);
       for (i=0; i<len; i++) {
 	const char *name = CHAR(STRING_ELT(names, i));
-	if (strcmp("xdimprev", name) == 0) {
+	if (STRCMP("xdimprev", name) == 0) {
 	  INTEGER(VECTOR_ELT(res, i))[0] = orig->xdimprev;
 	  break;
 	}
@@ -663,7 +661,7 @@ void Path(cov_model *cov, cov_model *sub) {
 void leer(int level){
   char format[255];
   if (level == 0) return;
-  sprintf(format,"%%%ds", -level * 3);
+  SPRINTF(format,"%%%ds", -level * 3);
   PRINTF(format, "");
 }
 
@@ -773,7 +771,7 @@ void pmi(cov_model *cov, char alles, int level, int maxlevel) {
   }
 
   cov_fct *CC = C;
-  while(strcmp(CC->name, InternalName) ==0) CC--;
+  while(STRCMP(CC->name, InternalName) ==0) CC--;
   if (level == 0) 
     PRINTF("******   %s, %s   ****** [%d,%d]", CC->nick, CC->name, cov->nr, cov->zaehler);
   else PRINTF("    **** %s, %s **** [%d,%d]", CC->nick, CC->name, cov->nr, cov->zaehler);  PRINTF("\n");
@@ -782,7 +780,7 @@ void pmi(cov_model *cov, char alles, int level, int maxlevel) {
 
   for(i=0; i<C->kappas; i++) {
     strcpy(name, 
-	   !strcmp(C->kappanames[i], FREEVARIABLE) && 
+	   !STRCMP(C->kappanames[i], FREEVARIABLE) && 
 	           cov->ownkappanames != NULL && cov->ownkappanames[i] != NULL
 	   ? cov->ownkappanames[i]
 	   : C->kappanames[i]
@@ -1294,6 +1292,7 @@ SEXP IGetModel(cov_model *cov, int modus, int spConform, bool solveRandom,
   cov_fct *C = CovList + cov->nr; // nicht gatternr
   bool plus_mixed_models;
 
+
   if ((cov->nr == NATSC_INTERN && modus != GETMODEL_AS_SAVED) ||
       (cov->nr == NATSC_USER && modus == GETMODEL_DEL_NATSC)) { 
     return IGetModel(cov->sub[0], modus, spConform, solveRandom, 
@@ -1307,7 +1306,7 @@ SEXP IGetModel(cov_model *cov, int modus, int spConform, bool solveRandom,
       nmodelinfo++;
   }
   for (i=0; i<C->kappas; i++) {
-    if ((PisNULL(i) || !strcmp(C->kappanames[i], INTERNAL_PARAM) ||
+    if ((PisNULL(i) || !STRCMP(C->kappanames[i], INTERNAL_PARAM) ||
 	 (do_notreturnparam && SortOf(cov, i, 0, 0) == DONOTRETURNPARAM))
 	&& cov->kappasub[i] == NULL)
       nmodelinfo--;
@@ -1336,7 +1335,13 @@ SEXP IGetModel(cov_model *cov, int modus, int spConform, bool solveRandom,
   }
 
   for(i=0; i<C->kappas; i++) {
-    if (!strcmp(C->kappanames[i], INTERNAL_PARAM)) continue;
+    //printf("donotreturnparam = %d\n", do_notreturnparam);
+
+    if ( (!STRCMP(C->kappanames[i], INTERNAL_PARAM) ||
+	  PisNULL(i) ||
+	  (do_notreturnparam && SortOf(cov, i, 0, 0) == DONOTRETURNPARAM))
+	 && cov->kappasub[i] == NULL
+      ) continue;
     if (cov->kappasub[i] != NULL && (!solveRandom || PisNULL(i))) {
       SET_STRING_ELT(nameMvec, k, mkChar(C->kappanames[i]));
       SET_VECTOR_ELT(model, k++,
@@ -1344,15 +1349,6 @@ SEXP IGetModel(cov_model *cov, int modus, int spConform, bool solveRandom,
 			       do_notreturnparam));
       continue;
     }
-    // naechste Zeile geloescht 30.4.2013
-    if (PisNULL(i) ||
-	(do_notreturnparam && SortOf(cov, i, 0, 0) == DONOTRETURNPARAM)) {	
-       // k++; // 9.1.09, needed for "$" (proj can be NULL) -- 19.1.09 k++ 
-	// deleted since otherwise outcome does not fit 
-	// model definition anymore (arbitrary NULL in the definition)
-	// instead: nmodelinfo--; above wihtin loop
-      continue;
-    }     
     SET_STRING_ELT(nameMvec, k, mkChar(C->kappanames[i]));
     SET_VECTOR_ELT(model, k++, 
 		   Param(cov, (void*) cov->px[i], cov->nrow[i],
@@ -1369,6 +1365,8 @@ SEXP IGetModel(cov_model *cov, int modus, int spConform, bool solveRandom,
     }
   }
 
+  //  printf("%s %d %d %d\n", NAME(cov), k, nmodelinfo, C->kappas);
+  //   PMI(cov);
   assert(k == nmodelinfo);
 
   setAttrib(model, R_NamesSymbol, nameMvec);
@@ -1577,7 +1575,7 @@ SEXP GetCoordSystem(SEXP keynr, SEXP oldsystem, SEXP newsystem) {
 
     if (n_s == coord_mix && GLOBAL.internal.warn_coord_change) {    
       char msg[300];
-      sprintf(msg, "the covariance model relies on at least two different coordinate systems. Check that this is not due to misspecification of the covariance model. To avoid this warning set 'RFoptions(%s=FALSE)'", 
+      SPRINTF(msg, "the covariance model relies on at least two different coordinate systems. Check that this is not due to misspecification of the covariance model. To avoid this warning set 'RFoptions(%s=FALSE)'", 
 	      internals[INTERNALS_COORD_CHANGE]);
       warning(msg);
       GLOBAL.internal.warn_coord_change = false;
@@ -1591,7 +1589,7 @@ SEXP GetCoordSystem(SEXP keynr, SEXP oldsystem, SEXP newsystem) {
     case reportcoord_warnings_orally : 
       if (warn) {
 	char msg[200];
-	sprintf(msg, "internal change of coordinate system from '%s' to '%s'. To avoid this message change ",
+	SPRINTF(msg, "internal change of coordinate system from '%s' to '%s'. To avoid this message change ",
 		COORD_SYS_NAMES[os], COORD_SYS_NAMES[n_s]);
 	warning(msg);
       } 
